@@ -1,4 +1,5 @@
 /*
+ * Copyright (c) 2013, Red Hat.
  * Copyright (c) 2007-2008, Aconex.  All Rights Reserved.
  * 
  * This program is free software; you can redistribute it and/or modify it
@@ -394,23 +395,32 @@ void ChartDialog::archiveButtonClicked()
 
 void ChartDialog::hostButtonClicked()
 {
-    HostDialog *h = new HostDialog(this);
-    int sts;
+    HostDialog *host = new HostDialog(this);
 
-    if (h->exec() == QDialog::Accepted) {
-	QString host = h->hostLineEdit->text().trimmed();
-	if ((sts = liveGroup->use(PM_CONTEXT_HOST, host)) < 0) {
-	    host.prepend(tr("Cannot connect to host: "));
-	    host.append(tr("\n"));
-	    host.append(tr(pmErrStr(sts)));
-	    QMessageBox::warning(this, pmProgname, host,
+    if (host->exec() == QDialog::Accepted) {
+	QString hostspec = host->getHostSpecification();
+	int sts, flags = host->getContextFlags();
+
+	if (hostspec == QString::null || hostspec.length() == 0) {
+	    hostspec.append(tr("Hostname not specified\n"));
+	    QMessageBox::warning(this, pmProgname, hostspec,
+		    QMessageBox::Ok|QMessageBox::Default|QMessageBox::Escape,
+		    Qt::NoButton, Qt::NoButton);
+	} else if ((sts = liveGroup->use(PM_CONTEXT_HOST, hostspec, flags)) < 0) {
+	    hostspec.prepend(tr("Cannot connect to host: "));
+	    hostspec.append(tr("\n"));
+	    hostspec.append(tr(pmErrStr(sts)));
+	    QMessageBox::warning(this, pmProgname, hostspec,
 		    QMessageBox::Ok|QMessageBox::Default|QMessageBox::Escape,
 		    Qt::NoButton, Qt::NoButton);
 	} else {
+	    console->post(PmChart::DebugUi,
+			"ChartDialog::newHost: %s (flags=0x%x)",
+			(const char *)hostspec.toAscii(), flags);
 	    setupAvailableMetricsTree(false);
 	}
     }
-    delete h;
+    delete host;
 }
 
 void ChartDialog::sourceButtonClicked()
@@ -626,7 +636,7 @@ void ChartDialog::rgbEd()
     sEd->setValue(my.sat);
     vEd->setValue(my.val);
     showCurrentColor();
-    emit newCol(my.currentColor);
+    Q_EMIT newCol(my.currentColor);
 }
 
 void ChartDialog::hsvEd()
@@ -641,7 +651,7 @@ void ChartDialog::hsvEd()
     gEd->setValue(qGreen(my.currentColor));
     bEd->setValue(qBlue(my.currentColor));
     showCurrentColor();
-    emit newCol(my.currentColor);
+    Q_EMIT newCol(my.currentColor);
 }
 
 void ChartDialog::showCurrentColor()
@@ -789,7 +799,7 @@ void ChartDialog::changeChartPlot(Chart *cp, NameSpace *name, int m)
     Chart::Style style = (Chart::Style)(typeComboBox->currentIndex() + 1);
     cp->setStroke(m, style, name->currentColor());
     cp->setLabel(m, name->label());
-    cp->revivePlot(m);
+    cp->reviveItem(m);
 }
 
 void ChartDialog::createChartPlot(Chart *cp, NameSpace *name)
@@ -797,9 +807,9 @@ void ChartDialog::createChartPlot(Chart *cp, NameSpace *name)
     Chart::Style style = (Chart::Style)(typeComboBox->currentIndex() + 1);
     pmMetricSpec pms;
 
-    const char *nlabel = NULL;
+    char *nlabel = NULL;
     if (name->label().isEmpty() == false)
-	nlabel = (const char *)name->label().toAscii();
+	nlabel = strdup((const char *)name->label().toAscii());
     pms.isarch = (name->sourceType() == PM_CONTEXT_LOCAL) ? 2 :
 		((name->sourceType() == PM_CONTEXT_ARCHIVE) ? 1 : 0);
     pms.source = strdup((const char *)name->sourceName().toAscii());
@@ -817,7 +827,7 @@ void ChartDialog::createChartPlot(Chart *cp, NameSpace *name)
 	pms.inst[0] = NULL;
     }
     cp->setStyle(style);
-    int m = cp->addPlot(&pms, nlabel);
+    int m = cp->addItem(&pms, nlabel);
     if (m < 0) {
 	QString	msg;
 	if (pms.inst[0] != NULL)
@@ -845,11 +855,12 @@ void ChartDialog::createChartPlot(Chart *cp, NameSpace *name)
 	free(pms.inst[0]);
     free(pms.metric);
     free(pms.source);
+    free(nlabel);
 }
 
 void ChartDialog::deleteChartPlot(Chart *cp, int m)
 {
-    cp->delPlot(m);
+    cp->removeItem(m);
 }
 
 void ChartDialog::setCurrentScheme(QString scheme)
