@@ -1,4 +1,5 @@
 /*
+ * Copyright (c) 2012, Red Hat.
  * Copyright (c) 2006, Ken McDonell.  All Rights Reserved.
  * Copyright (c) 2006-2009, Aconex.  All Rights Reserved.
  * 
@@ -11,13 +12,11 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- * 
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  */
+#include <QtCore/QUrl>
 #include <QtCore/QTimer>
 #include <QtCore/QLibraryInfo>
+#include <QtGui/QDesktopServices>
 #include <QtGui/QDesktopWidget>
 #include <QtGui/QApplication>
 #include <QtGui/QPrintDialog>
@@ -44,7 +43,6 @@
 
 PmChart::PmChart() : QMainWindow(NULL)
 {
-    my.assistant = NULL;
     my.dialogsSetup = false;
     setIconSize(QSize(22, 22));
 
@@ -142,8 +140,6 @@ void PmChart::quit()
 	my.saveview->reject();
 	my.settings->reject();
     }
-    if (my.assistant)
-	my.assistant->closeAssistant();
     if (pmtime)
 	pmtime->quit();
     pmflush();
@@ -226,9 +222,7 @@ void PmChart::updateToolbarLocation()
 	addToolBar(Qt::RightToolBarArea, toolBar);
     else
 	addToolBar(Qt::TopToolBarArea, toolBar);
-#if QT_VERSION >= 0x040300
     setUnifiedTitleAndToolBarOnMac(globalSettings.nativeToolbar);
-#endif
 }
 
 void PmChart::updateToolbarOrientation(Qt::Orientation orientation)
@@ -394,38 +388,18 @@ void PmChart::fileQuit()
     QApplication::exit(0);
 }
 
-void PmChart::assistantError(const QString &msg)
-{
-    QMessageBox::warning(this, pmProgname, msg);
-}
-
-void PmChart::setupAssistant()
-{
-    if (my.assistant)
-	return;
-    my.assistant = new QAssistantClient(
-		QLibraryInfo::location(QLibraryInfo::BinariesPath), this);
-    connect(my.assistant, SIGNAL(error(const QString &)),
-    		    this, SLOT(assistantError(const QString &)));
-
-    QString documents = HTMLDIR;
-    QString separator = QString(__pmPathSeparator());
-    documents.append(separator).append("html");
-    documents.append(separator).append("pcpdoc.adp");
-
-    QStringList arguments;
-    arguments << "-profile" << documents;
-    my.assistant->setArguments(arguments);
-}
-
 void PmChart::helpManual()
 {
-    setupAssistant();
-    QString documents = HTMLDIR;
+    bool ok;
+    QString documents("file://" HTMLDIR);
     QString separator = QString(__pmPathSeparator());
     documents.append(separator).append("html");
     documents.append(separator).append("index.html");
-    my.assistant->showPage(documents);
+    ok = QDesktopServices::openUrl(QUrl(documents, QUrl::TolerantMode));
+    if (!ok) {
+	documents.prepend("Failed to open:\n");
+	QMessageBox::warning(this, pmProgname, documents);
+    }
 }
 
 void PmChart::helpAbout()
@@ -434,6 +408,11 @@ void PmChart::helpAbout()
     about.exec();
 }
 
+void PmChart::helpAboutQt()
+{
+    QApplication::aboutQt();
+}
+
 void PmChart::helpSeeAlso()
 {
     SeeAlsoDialog seealso(this);
@@ -852,9 +831,9 @@ void PmChart::setupEnabledActionsList()
 				// separator
 			  << zoomInAction << zoomOutAction
 				// separator
-			  << fileExportAction << filePrintAction
+			  << fileExportAction
 				// separator
-			  << newPmchartAction;
+			  << timeControlAction << newPmchartAction;
 
     if (globalSettings.toolbarActions.size() > 0) {
 	setEnabledActionsList(globalSettings.toolbarActions, false);
