@@ -1,5 +1,6 @@
 /*
  * Copyright (c) 2006, Ken McDonell.  All Rights Reserved.
+ * Copyright (c) 2013, Red Hat Inc.
  * 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -453,6 +454,8 @@ new_chart:
 		    }
 		    if (strcasecmp(w, "plot") == 0)
 			style = Chart::LineStyle;
+		    else if (strcasecmp(w, "line") == 0)
+			style = Chart::LineStyle;
 		    else if (strcasecmp(w, "bar") == 0)
 			style = Chart::BarStyle;
 		    else if (strcasecmp(w, "stacking") == 0)
@@ -461,6 +464,8 @@ new_chart:
 			style = Chart::AreaStyle;
 		    else if (strcasecmp(w, "utilization") == 0)
 			style = Chart::UtilisationStyle;
+		    else if (strcasecmp(w, "event") == 0)
+			style = Chart::EventStyle;
 		    else {
 			xpect("<chart style>", w);
 			goto abort_chart;
@@ -1003,7 +1008,7 @@ done_tab:
 			numinst = pmGetInDomArchive(desc.indom, &instlist, &namelist);
 		    else
 			numinst = pmGetInDom(desc.indom, &instlist, &namelist);
-		    if (numinst < 1) {
+		    if (numinst < 0) {
 			QString	msg = QString();
 			msg.sprintf("\nMetric \"%s\" for\n%s %s: empty instance domain",
 			    pms.metric, pms.isarch ? "archive" : "host",
@@ -1021,7 +1026,7 @@ done_tab:
 			    goto skip;
 			}
 		    }
-		    pms.ninst = 1;
+		    pms.ninst = numinst ? 1 : 0;
 		    if (pms.inst[0] != NULL) {
 			free(pms.inst[0]);
 			pms.inst[0] = NULL;
@@ -1070,11 +1075,11 @@ try_plot:
 		    strcat(tmp, pms.inst[0]);
 		    w +=2;
 		    strcat(tmp, w);
-		    m = cp->addPlot(&pms, tmp);
+		    m = cp->addItem(&pms, tmp);
 		    free(tmp);
 		}
 		else
-		    m = cp->addPlot(&pms, legend);
+		    m = cp->addItem(&pms, legend);
 		if (m < 0) {
 		    if (!optional) {
 			QString	msg;
@@ -1207,9 +1212,6 @@ void SaveViewDialog::saveChart(FILE *f, Chart *cp, bool hostDynamic)
     if (s != NULL)
 	fprintf(f, " title \"%s\"", s);
     switch (cp->style()) {
-	case Chart::NoStyle:
-	    s = "none - botched in Save!";
-    	    break;
 	case Chart::LineStyle:
 	    s = "plot";
 	    break;
@@ -1225,6 +1227,13 @@ void SaveViewDialog::saveChart(FILE *f, Chart *cp, bool hostDynamic)
 	case Chart::UtilisationStyle:
 	    s = "utilization";
 	    break;
+	case Chart::EventStyle:
+	    s = "event";
+	    break;
+	case Chart::NoStyle:
+	default:
+	    s = "none";
+	    break;
     }
     fprintf(f, " style %s", s);
     if (cp->style() != Chart::UtilisationStyle) {
@@ -1238,9 +1247,8 @@ void SaveViewDialog::saveChart(FILE *f, Chart *cp, bool hostDynamic)
 	fprintf(f, " antialiasing off");
     fputc('\n', f);
     for (int m = 0; m < cp->metricCount(); m++) {
-	char	*p, *q, *qend;
-
-	if (cp->activePlot(m) == false)
+	char	*p;
+	if (cp->activeItem(m) == false)
 	    continue;
 	fprintf(f, "\tplot");
 	p = cp->legendSpec(m);
@@ -1250,22 +1258,10 @@ void SaveViewDialog::saveChart(FILE *f, Chart *cp, bool hostDynamic)
 	if (hostDynamic == false)
 	    fprintf(f, " host %s", (const char *)
 			cp->metricContext(m)->source().host().toAscii());
-	p = (char *)(const char *)cp->name(m).toAscii();
-	if ((q = strchr(p, '[')) != NULL) {
-	    // metric with an instance
-	    if ((qend = strrchr(q, ']')) == NULL) {
-		QString	msg;
-		msg.sprintf("Botch @ metric name: \"%s\"", p);
-		err(E_CRIT, false, msg);
-	    }
-	    else {
-		*q++ = '\0';
-		*qend = '\0';
-		fprintf(f, " metric %s instance \"%s\"", p, q);
-	    }
-	}
-	else // singular metric
-	    fprintf(f, " metric %s", p);
+        fprintf(f, " metric %s", (const char *)
+                cp->metricName(m).toAscii());
+	if (cp->metric(m)->explicitInsts())
+            fprintf(f, " instance \"%s\"", (const char*)cp->metricInstance(m).toAscii());
 	fputc('\n', f);
     }
 }
@@ -1277,7 +1273,7 @@ bool SaveViewDialog::saveView(QString file, bool hostDynamic,
     int		c, t;
     Tab		*tab;
     Gadget	*gadget;
-    const char	*path = (const char *)file.toAscii();
+    char	*path = strdup((const char *)file.toAscii());
     QStringList	schemes;
 
     if ((f = fopen(path, "w")) == NULL)
@@ -1324,11 +1320,13 @@ bool SaveViewDialog::saveView(QString file, bool hostDynamic,
 
     fflush(f);
     fclose(f);
+    free (path);
     return true;
 
 noview:
     QString errmsg;
     errmsg.sprintf("Cannot open \"%s\" for writing\n%s", path, strerror(errno));
     err(E_CRIT, false, errmsg);
+    free (path);
     return false;
 }
